#include "Cgiu.h"
#ifdef VxWorks
#include "timers.h"
#else
#include "time.h"
#endif

#ifdef VxWorks
   #define SERIALDEVICENAME "/tyCo/0"
#else
   #define SERIALDEVICENAME "/dev/ttyS0"
#endif



/*------------------------------Math functions---------------------------------*/

void cyberGraspDrv_dglove_22_to_24_sensors ( unsigned char *buf );

void cyberGraspDrv_dglove_18_to_24_sensors ( unsigned char *buf );

void convertForces(float* forces, unsigned char* result);



/*------------------------------Cgiu functions---------------------------------*/
 
// Constructor
Cgiu::Cgiu() : _fsm(*this),
               m_serial()
{
  m_sensorData24 = new unsigned char [24];
  for (int i = 0; i < 5; i++) m_fingerForces[i] = 0.0;
}


// Destructor
Cgiu::~Cgiu() 
{
  delete(m_sensorData24);
}


/*------------------------------Public functions-------------------------------*/

int Cgiu::initialize()
{ 
  // Open the port
  if (m_serial.openPort(SERIALDEVICENAME, 38400, 8) == -1) 
		{
    printf("Alert: Error opening serial device!\n");
    return -1;
  }
  // Get the num of sensors
  m_numOfSensors = GetNumOfSensors();
  if ((m_numOfSensors != 18) && (m_numOfSensors != 22)) 
		{
    printf("Alert: Error detecting number of sensors!\n");	
    return -1;	
  }
  printf("Number of Sensors: %d\n", m_numOfSensors );
  
  // Grasp attached?
  m_graspDetected = CheckIfGraspDetected();
  if(m_graspDetected == -1) 
	 {
   printf("Alert: Error checking if Grasp connected!\n");
   return -1;
  }
  return 1;
}


void Cgiu::GetSensorData() 
{
  int bytesToRead;
  m_isTimeout = 0;
  m_wrongAnswer = 0;
  unsigned char GET_VALUES[1] = {'G'};
  // Send the command
  m_serial.writeData(GET_VALUES, 1);
   
  // If Grasp is attached 48 bytes are returned no matter which glove is used.
  if (m_graspDetected == 1)
     bytesToRead = 48; 
  else 
    bytesToRead = m_numOfSensors + 2;
   
  // Read the data byte by byte
  int check = m_serial.readByteByByte(bytesToRead); 

		// If timeout, set the variable and exit function 
  if (check == -1) 
  { 
    m_isTimeout = 1; 
    return;
   }

  // If wrong data is received, exit function
  if (!(m_serial.m_readBuffer[0] == 'G')) 
  { 
    m_wrongAnswer = 1; 
    return;
  }

  // On success copy the new sensor data in the member variable
  for (int i = 1; i <= m_numOfSensors; i++) 
    m_sensorData24[i-1] = m_serial.m_readBuffer[i];
  
		// Transform the values into a 24 byte buffer
  if (m_numOfSensors == 18) 
  {
	   cyberGraspDrv_dglove_18_to_24_sensors(m_sensorData24);
  } 
  else 
  {  
     cyberGraspDrv_dglove_22_to_24_sensors(m_sensorData24);
  }
}


void Cgiu::GetLampStatus() 
{
  m_isTimeout = 0;
  m_wrongAnswer = 0;  
  unsigned char GET_LAMP_STATUS[2] = {'?', 'L'};
 
  // Send the command
  m_serial.writeData(GET_LAMP_STATUS, 2);
  // Check the answer, fails if timeout or wrong answer 
  int check = m_serial.readByteByByte(4);
  if (check == -1)
  { 
    m_isTimeout = 1; 
    return;
  }
  if (!((m_serial.m_readBuffer[0] == '?') &&
        (m_serial.m_readBuffer[1] == 'L')))
  { 
    m_wrongAnswer = 1; 
    return;
  }
  // On success copy the lamp status (0 or 1) into the m_lampStatus variable
  m_lampStatus = (int)m_serial.m_readBuffer[2];
}


void Cgiu::SwitchLampStatus() 
{
  unsigned char newLampStatus = m_lampStatus - 1;
  unsigned char SET_LAMP_STATUS[6] = {'L', newLampStatus, 'J', 1, 'W',
                                     newLampStatus};
  // Send the command 
  m_serial.writeData(SET_LAMP_STATUS, 6);
  // Check the answer
  int check = m_serial.readByteByByte(6);
  // On read error set m_isTimeout and exit
  if (check == -1)
  { 
    m_isTimeout = 1; 
    return;
  }
  // Check if the right answer was returned from the glove
  unsigned char CHECK_ANSWER[6] = {'L', 0, 'J', 0, 'W', 0 };
  for (int i=0; i < 6; i++)
  {
    if (m_serial.m_readBuffer[i] != CHECK_ANSWER[i])
    {
      m_wrongAnswer = 1;
      return;
    }
  }
}


void Cgiu::SendFingerForces() 
{
  unsigned char force_converted[10];
  // Convert the forces
  convertForces(m_fingerForces, force_converted);
  // Send the command and the forces
  m_serial.writeData((unsigned char*)"X", 1);
  m_serial.writeData(force_converted, 10);
}


void Cgiu::TimeoutAlert() 
{
  printf("\nWarning: Timeout occured while reading data. ");
}


void Cgiu::WrongAnswerAlert() 
{
  printf("\nWarning: Wrong data was returned from the data glove. ");
}


void Cgiu::Reset() {
  m_serial.flushPort();
  printf("RESET: Clear the port! (tcflush))\n");
}


void Cgiu::UnacceptableAlert() {
  printf("\nWarning: Transition isn't valid.\n");
}


/*------------------------------Private functions------------------------------*/

int Cgiu::GetNumOfSensors() 
{
  unsigned char GET_NUM_OF_SENSORS[2] = {'?', 'S'};
  // Send the command
  m_serial.writeData(GET_NUM_OF_SENSORS, 2);
  // Read the answer
  int res = m_serial.readByteByByte(4);
  if (res != 4) 
		  return -1; 
  else 
    return m_serial.m_readBuffer[2];
}


int Cgiu::CheckIfGraspDetected()
{
   unsigned char QUERY_STATUS[2] = {'?', 'V'};   
   m_serial.writeData(QUERY_STATUS, 2);

   {
      timespec myTime;
      myTime.tv_sec = 2;
      myTime.tv_nsec = 0;      
      nanosleep(&myTime,NULL);
   }
   
   unsigned char* tmp = new unsigned char [255];
   int check = m_serial.readData(tmp, 255);
   switch(check) {
      case 5 : return 1; break;
      case 7 : return 0; break;
      default: return -1;
   }
}



/*------------------------------Math functions-------------------------------*/

void cyberGraspDrv_dglove_18_to_24_sensors (unsigned char *buf) {
  if (buf == NULL) 
    return;
  buf[23] = 60;
  buf[22] = buf[17];
  buf[21] = buf[16];
  buf[20] = buf[15];
  buf[19] = buf[14];
  buf[18] = 101;
  buf[17] = buf[13];
  buf[16] = buf[12];
  buf[15] = buf[11];
  buf[14] = 81;
  buf[13] = buf[10];
  buf[12] = buf[9];
  buf[11] = buf[8];
  buf[10] = 91;
  buf[9] = buf[7];
  buf[8] = buf[6];
  buf[7] = 125;
  buf[6] = 65;
}


void cyberGraspDrv_dglove_22_to_24_sensors (unsigned char *buf) {
  int i;
  if (buf == NULL) 
    return;
  buf[23] = 60;
  for (i = 22; i > 7; i--) buf[i] = buf[i-1];
  buf[7] = 125;
}


void convertForces(float* forces, unsigned char* result)  {
  int intForceVal;
  for (int i = 0; i < 5; i++) {
     intForceVal = (int) (32767 * forces[i]);
     result[2*i] = (char)(intForceVal & 0x00FF);
     result[(2*i)+1] = (char)((intForceVal >> 8) & 0x00FF);
  }
}


/*-------------------------------------------------------------------------------
-------------------------------------------------------------------------------*/


