#ifndef _H_CGIU_SM
#define _H_CGIU_SM

#include <statemap.h>

// Forward declarations.
class Map1;
class Map1_WaitForCmd;
class Map1_DecisionState;
class Map1_Error;
class Map1_Default;
class CgiuState;
class CgiuContext;
class Cgiu;

class CgiuState : public statemap::State
{
public:
    CgiuState(const char *name, int id)
    : statemap::State(name, id)
    {};

    virtual void Entry(CgiuContext&) {};
    virtual void Exit(CgiuContext&) {};

    virtual void CheckForErrors(CgiuContext& context);
    virtual void GetData(CgiuContext& context);
    virtual void GetLamp(CgiuContext& context);
    virtual void SendForces(CgiuContext& context);
    virtual void SwitchLamp(CgiuContext& context);

protected:
    virtual void Default(CgiuContext& context);
};

class Map1
{
public:
    static Map1_WaitForCmd    WaitForCmd;
    static Map1_DecisionState    DecisionState;
    static Map1_Error    Error;

private:
    Map1() {};
};

class Map1_Default : public CgiuState
{
public:
    Map1_Default(const char *name, int id)
    : CgiuState(name, id)
    {};

    virtual void GetData(CgiuContext& context);
    virtual void GetLamp(CgiuContext& context);
    virtual void SwitchLamp(CgiuContext& context);
    virtual void CheckForErrors(CgiuContext& context);
    virtual void SendForces(CgiuContext& context);
};

class Map1_WaitForCmd : public Map1_Default
{
public:
    Map1_WaitForCmd(const char *name, int id)
    : Map1_Default(name, id)
    {};

    void GetData(CgiuContext& context);
    void GetLamp(CgiuContext& context);
    void SendForces(CgiuContext& context);
    void SwitchLamp(CgiuContext& context);
};

class Map1_DecisionState : public Map1_Default
{
public:
    Map1_DecisionState(const char *name, int id)
    : Map1_Default(name, id)
    {};

    void CheckForErrors(CgiuContext& context);
};

class Map1_Error : public Map1_Default
{
public:
    Map1_Error(const char *name, int id)
    : Map1_Default(name, id)
    {};

    void GetData(CgiuContext& context);
    void GetLamp(CgiuContext& context);
    void SwitchLamp(CgiuContext& context);
};

class CgiuContext : public statemap::FSMContext
{
public:
    CgiuContext(Cgiu& owner)
    : _owner(owner)
    {
        setState(Map1::WaitForCmd);
        Map1::WaitForCmd.Entry(*this);
    };

    Cgiu& getOwner() const
    {
        return(_owner);
    };

    CgiuState& getState() const
    {
        if (_state == NULL)
        {
            throw statemap::StateUndefinedException();
        }

        return (dynamic_cast<CgiuState&>(*_state));
    };

    void CheckForErrors()
    {
        (getState()).CheckForErrors(*this);
    };

    void GetData()
    {
        (getState()).GetData(*this);
    };

    void GetLamp()
    {
        (getState()).GetLamp(*this);
    };

    void SendForces()
    {
        (getState()).SendForces(*this);
    };

    void SwitchLamp()
    {
        (getState()).SwitchLamp(*this);
    };

private:
    Cgiu& _owner;
};

#endif
