#ifndef SERIALPORTDRIVER_H
#define SERIALPORTDRIVER_H

#include <unistd.h>


// Default timeout (1.5 sec) for reading data from the device
const int TIMEOUT = 15; 


/*
 * Class allows easy dealing with a device.
	* There are functions for reading and writing on a device.
	* It also offers methods for opening and initializing the device first.
	* Dependent on the system (Linux or VXWorks) different implementations of the
 * functions are called.
 */
class SerialPortDriver
{
								
public:
 
  // Constructor
	 SerialPortDriver();

	 // Destructor
  ~SerialPortDriver();


/*------------------------------Public functions------------------------------*/   

  int openPort(char* device, long int baudrate, int parity);  

  int writeData(unsigned char* buf, int num);

  int readData(unsigned char* buf, int num);
	
  int readByteByByte(int num, int timeout); 
  int readByteByByte(int num) 
  {
    return readByteByByte(num, TIMEOUT);
  }
 
 
  void flushPort();

  unsigned char* getReadBuffer()
  {
    return m_readBuffer;
  }


/*------------------------------Public members---------------------------------*/

  // readByteByByte() fills buffer with values from the data glove
  unsigned char* m_readBuffer;
 


private:

/*------------------------------Private functions------------------------------*/

  void setTimeout(int timeout); 

  void setPortParameters(long int baudrate, int parity); 



/*------------------------------Private members--------------------------------*/
 
  // File descriptor, important for reading and writing on the device
  int             m_fd;

  // Time limit while reading from the port
  int             m_actTimeout; 
	
};

void sleeper(int sec, long int nsec);


#endif /* SERIALPORTDRIVER_H */


/*-------------------------------------------------------------------------------
-------------------------------------------------------------------------------*/
